clear
clc

addpath ('D:\matlab\casadi-matlabR2014b-v3.2.3')
addpath('D:\matlab')
import casadi.*

Ts = 1; % Sample time
y = MX.sym('y'); 
u = MX.sym('u'); 
tau = (174/Ts);

dx1 = ((-0.1*u^2+4*u+5) - y)/tau;

ode = struct('x',y,'p',u,'ode',dx1,'quad',y);  
opts = struct('tf',Ts);

F = integrator('F','cvodes',ode,opts); 

xf = 42.5;
u_in0 = 15;
u_in = u_in0;

nIter = 4*1e5;

h = waitbar(0,'Simulation in Progress...');

TD = 800;                % dither time constant
Th = (0.2)*TD;           % High Pass Filter time constant
Tl = 10*TD;              % Low Pass Filter time constant
a = 1;                   % sine amplitude 
Ki = 0.005;              % Controller gain

%intitalisation
z(1) = 0;
x(1) = 0;
u_esc(1) = 15;
y_esc(1) = 42.5;
sim.u(1) = 15;
sim.y(1) = 42.5;
J(1) = y_esc(1);
u_hat(1) = u_esc(1);
sine(1) = 0;
f = 1/TD;

% filter coefficients
al = Ts/(Ts + Tl);
ah = Th/(Ts + Th);

ARX = 1;
pbrs = 0;


for sim_k  = 2:nIter
    waitbar(sim_k /nIter,h,sprintf('Time: %0.0f min',sim_k*Ts/60))
    
    Fk = F('x0',xf,'p',u_in);
    xf = full(Fk.xf);
    
    sim.y(sim_k) = xf + 0.005*randn(1);
    sim.u(sim_k) = u_in + 0.005*randn(1);
    
    y_esc(sim_k) = sim.y(sim_k);
    J(sim_k) = sim.y(sim_k);
    u_esc(sim_k) = sim.u(sim_k);
   
    z(sim_k) = ah*z(sim_k-1) + ah*J(sim_k) - ah*J(sim_k-1);               % High pass filter
    x(sim_k) = ((1-al)*x(sim_k-1) + al*z(sim_k)*sin(2*pi*f*Ts*(sim_k)));  % correlation and Low pass filter
    u_esc(sim_k) = u_esc(sim_k-1) + Ts*Ki*x(sim_k).*2/a;                  % I-controller
    u_hat(sim_k) = u_esc(sim_k) + a*sin(2*pi*f*Ts*(sim_k));               % estimated optimal input + dither
    sine(sim_k)= sin(2*pi*f*Ts*(sim_k));
    
    u_in = u_hat(sim_k);
    

end
close(h);
%% Plotting

hold all
figure(1)
subplot(311)
hold all
plot(y_esc)
set(gca,'FontSize',20);
ylabel 'y'

subplot(312)
hold all
plot(u_hat)
set(gca,'FontSize',20);
ylabel 'u'

subplot(313)
hold all
plot(x.*2/a)
set(gca,'FontSize',20);
ylabel 'J_u'

