clear
clc

addpath ('D:\matlab\casadi-matlabR2014b-v3.2.3')
addpath('D:\matlab')
import casadi.*

Ts = 1; % Sample time
y = MX.sym('y'); 
u = MX.sym('u'); 
x1 = MX.sym('x1');
x2 = MX.sym('x2');
tau = (174/Ts);

tau_1 = 174;
tau_2 = 60;
tau_a = 40;                 % tau_a = 0 and 40 are studied

[a, b, c, d] = tf2ss([0 tau_a 1],[tau_1*tau_2 tau_1+tau_2 1]);


% dx/dt=Ax+Bu, y=Cx+Du, x=[x1 x2]', A=2X2 matrix
dx1 = a(1,1)*x1+a(1,2)*x2+b(1,1)*(-0.1*u^2+4*u+5);
dx2 = a(2,1)*x1+a(2,2)*x2+b(2,1)*(-0.1*u^2+4*u+5);
y = c(1,1)*x1+c(1,2)*x2+d*(-0.1*u^2+4*u+5);

ode = struct('x',vertcat(x1,x2),'p',u,'ode',vertcat(dx1,dx2),'quad',y);  
opts = struct('tf',Ts);

F = integrator('F','cvodes',ode,opts); 

xf = [0; 42.5/c(1,2)];
u_in0 = 15;
u_in = u_in0;

Ki = 0.005;                 % Controller gain
PRBS = 30;                  % PRBS calculation steps
l = 4*180;                  % Window size

nIter = 4*3600;

h = waitbar(0,'Simulation in Progress...');
aL = Ts/(Ts+ 100);
sim.uf = u_in;
sim.yf = c*xf;



ARX = 1;
pbrs = 0;

for sim_k  = 1:nIter
    waitbar(sim_k /nIter,h,sprintf('Time: %0.0f min',sim_k*Ts/60))
    
    Fk = F('x0',xf,'p',u_in);
    xf = full(Fk.xf);
    
    sim.y(sim_k) = c*xf + 0.005*randn(1);
    sim.u(sim_k) = u_in + 0.005*randn(1);
    sim.ySS(sim_k) = -0.1*u_in^2+4*u_in+5;
    sim.JuSS(sim_k) = -0.2*u_in+4;
    if sim_k>1
        sim.uf(sim_k) = (1-aL)*sim.uf(sim_k-1) + aL*sim.u(sim_k);
        sim.yf(sim_k) = (1-aL)*sim.yf(sim_k-1) + aL*sim.y(sim_k);
    end
    if sim_k >Gradient       
        ymeas = sim.yf(sim_k-Gradient:sim_k)';
        umeas = sim.uf(sim_k-Gradient:sim_k)';
        
        Ju(Gradient) = 1;           %initialization
        
        if Ju(sim_k-1) > 0.030      % ARX model
            Y0 = ymeas - mean(ymeas);
            U0 = umeas - mean(umeas);
            
            data = iddata(Y0,U0,Ts);            
            sysARX = arx(data,[2,2,0]);            
            sysD = idss(sysARX);            
            sys = d2c(sysD);
            Ju_hat = (-sys.C*(sys.A\sys.B) + sys.D);
            Ju(sim_k) = Ju_hat;
            flag(sim_k) = 1;
        else                        % LS method
            Y = ymeas;
            U = umeas;
            X = [U ones(size(U))];
            b = (inv(X'*X))*(X'*Y);
            Ju_hat = b(1,1);
            Ju(sim_k) = Ju_hat;
            flag(sim_k) = 0;
        end
        
        if sim_k > Gradient && (mean(Ju(sim_k-10:sim_k)) >0.001)   % I-controller
            u_in0 =   u_in0  +Ki*Ju_hat ;
        else
            u_in0 = u_in0 ;
        end
    end

if rem(sim_k,PRBS)==0
    pbrs = 1*idinput(1);
end
u_in = u_in0+ pbrs;  % add perturbation
end
close(h);

sim.Ju = Ju;

%%

figure(1)
subplot(411)
hold all
plot(sim.y)
plot(sim.yf)
plot( sim.ySS,'--')
set(gca,'FontSize',20);
grid on
ylabel 'y'

subplot(412)
hold all
plot(sim.u)
plot(sim.uf,'--')
set(gca,'FontSize',20);
grid on
ylabel 'u'

subplot(413)
hold all
plot(sim.JuSS,'--')
plot(sim.Ju)
set(gca,'FontSize',20);
grid on
ylabel 'J_u'
ylim([-2 2])

subplot(414)
hold all
plot(flag)
set(gca,'FontSize',20);
grid on
ylabel 'flag'
